from research_framework.base.plugin.base_plugin import BasePlugin
from research_framework.base.wrappers.filter_wrapper import BaseFilterWrapper
from research_framework.container.container import Container
from research_framework.lightweight.lightweight import FlyWeight
from research_framework.lightweight.model.item_model import ItemModel
from research_framework.pipeline.model.pipeline_model import PipelineModel, FilterModel, InputFilterModel
from research_framework.pipeline.pipeline import FitPredictPipeline

import pytest
import pandas as pd

test_pipeline = PipelineModel(
    name='pipeline para tests',
    train_input= 
        InputFilterModel(
            clazz='CSVPlugin',
            name='texts_depression_2018.csv',
            params={
                "filepath_or_buffer":"test/data/texts_depression_2018.csv",
                "sep": ",",
                "index_col": 0,
            },
        )
    ,
    test_input =
        InputFilterModel(
            clazz='CSVPlugin',
            name='texts_depression_2022.csv',
            params={
                "filepath_or_buffer":"test/data/texts_depression_2022.csv",
                "sep": ",",
                "index_col": 0,
            }
        )
    ,
    filters= [
        FilterModel(
            clazz="FilterRowsByNwords",
            params={
                "upper_cut": 100,
                "lower_cut": 10,
            }
        ),
        FilterModel(
            clazz="Tf",
            params={
                "lowercase":True
            }
        )
    ],
    metrics = [
        
    ]
)

@pytest.fixture
def test_simple_pipeline():
    print(Container.BINDINGS)
    Container.fly = FlyWeight()
    pipeline = FitPredictPipeline(test_pipeline)
    pipeline.start()
    return pipeline


def aux_delete_pipeline_generated_items(pipeline):
    print("- Train data:")
    for item in test_pipeline.train_input.items:
        print(item)
        assert Container.fly.unset_item(item.hash_code)
        
    print("- Test data:")
    for item in test_pipeline.test_input.items:
        print(item)
        assert Container.fly.unset_item(item.hash_code)
    
    print("- Trained models:")
    for plugin_filter in pipeline.pipeline.filters:
        if not plugin_filter.item is None:
            print(plugin_filter.item)
            assert Container.fly.unset_item(plugin_filter.item.hash_code)

@pytest.fixture
def delete_pipeline_items(test_simple_pipeline, request):
    request.addfinalizer(lambda: aux_delete_pipeline_generated_items(test_simple_pipeline))
    return test_simple_pipeline


def test_stored_items_types_and_wrappers(delete_pipeline_items):
    pipeline = delete_pipeline_items
    for item in test_pipeline.train_input.items:
        assert type(item) == ItemModel
        obj = Container.fly.get_wrapped_data_from_item(item)
        assert issubclass(type(obj), BaseFilterWrapper)
        obj2 = Container.fly.get_data_from_item(item)
        assert type(obj2) == pd.DataFrame

    for item in test_pipeline.test_input.items:
        assert type(item) == ItemModel
        obj = Container.fly.get_wrapped_data_from_item(item)
        assert issubclass(type(obj), BaseFilterWrapper)
        obj2 = Container.fly.get_data_from_item(item)
        assert type(obj2) == pd.DataFrame

    for plugin_filter in pipeline.pipeline.filters:
        if not plugin_filter.item is None:
            item = plugin_filter.item
            assert type(item) == ItemModel
            obj = Container.fly.get_wrapped_data_from_item(item)
            assert issubclass(type(obj), BaseFilterWrapper)
            obj2 = Container.fly.get_data_from_item(item)
            assert issubclass(type(obj2), BasePlugin)
    